@@ -19,7 +19,8 @@
 #include "exsltconfig.h"
 #include "exslt.h"
 
-const char *exsltLibraryVersion = LIBEXSLT_VERSION_STRING;
+const char *exsltLibraryVersion = LIBEXSLT_VERSION_STRING 
+				LIBEXSLT_VERSION_EXTRA;
 const int exsltLibexsltVersion = LIBEXSLT_VERSION;
 const int exsltLibxsltVersion = LIBXSLT_VERSION;
 const int exsltLibxmlVersion = LIBXML_VERSION;
@@ -32,6 +33,9 @@ const int exsltLibxmlVersion = LIBXML_VERSION;
 void
 exsltRegisterAll (void) {
     exsltCommonRegister();
+#ifdef EXSLT_CRYPTO_ENABLED
+    exsltCryptoRegister();
+#endif
     exsltMathRegister();
     exsltSetsRegister();
     exsltFuncRegister();
